""""
(c) 2020 Amazon Web Services, Inc. or its affiliates. All Rights Reserved. This AWS Content is provided subject to the terms of the AWS Customer
Agreement available at https://aws.amazon.com/agreement/ or other written agreement between Customer and Amazon Web Services, Inc.

EMR Security Certification
Description: The function checks for compliance on activities that could generate violations.
Checks for compliance on activities that could generate violations
Checks if an EMR cluster tagging follows the standard policies for service tagging - Cloud Tags

API Triggers: RunJobFlow,CreateTags,AddTags,RemoveTags
Services: Cloudwatch Events (trigger), EMR, SNS
Python 3.6 
"""

import os
import json
import boto3
import logging
from botocore.exceptions import ClientError

EMR_CLIENT = boto3.client('emr')
OUTBOUND_TOPIC_ARN = os.environ["outbound_topic_arn"]
REQUIRED_KEYS = os.environ["required_tagging_keys"].split(",")


def lambda_handler(event, context):
    """
    Lambda event handler that processes event received
    """
    setup_logging()
    log.info('Got an event!')
    log.info(event)

    if "errorMessage" in event["detail"]["eventName"]:
        print("Event carried an error message. Aborting")
        return
    if event["detail"]["eventName"] in ["RunJobFlow"] or ["CreateTags"] or ["AddTags"] or ["RemoveTags"]:
        non_compliance_tags = []
        # Checks if an EMR cluster tagging follows the standard policies for service tagging
        # Then runs the tags compliance checks
        if is_tag(event):
            non_compliance_tags = validate_tags(
                describe_emr_tags(get_emr_cluster_id(event)))
        if non_compliance_tags:
            subject = "Violation - EMR does not meet the tagging policy!"
            message = create_non_compliance_message(
                non_compliance_tags, get_emr_cluster_id(event))
            send_violation(OUTBOUND_TOPIC_ARN, message,
                           subject, event, context)
        else:
            print("No violations for EMR tagging policy found")

def get_emr_cluster_id(event):
    """
    The function checks the event of EMR and returns the cluster ID.
    """
    if event["detail"]["eventName"] == "RunJobFlow":
        emr_cluster_id = event["detail"]["responseElements"]["jobFlowId"]
    if event["detail"]["eventName"] == "CreateTags":
        for tag in event["detail"]["requestParameters"]["tagSet"]["items"]:
            if tag["key"] == "aws:elasticmapreduce:job-flow-id":
                emr_cluster_id = tag["value"]
    if event["detail"]["eventName"] == "AddTags":
        emr_cluster_id = event["detail"]["requestParameters"]["resourceId"]
    if event["detail"]["eventName"] == "RemoveTags":
        emr_cluster_id = event["detail"]["requestParameters"]["resourceId"]
    return emr_cluster_id

def describe_emr_tags(emr_cluster_id):
    """
    The function describe the EMR cluster and return the tags key and value.
    """
    try:
        describe_cluster = EMR_CLIENT.describe_cluster(
            ClusterId=emr_cluster_id
        )
        cluster_tags = (describe_cluster["Cluster"]["Tags"])
        return cluster_tags
    except ClientError as err:
        log.error(err)

def remove_whitespace_from_list_of_strings(string_list):
    """
    The function remove trailing whitespace from a list of strings
    """
    for index, item in enumerate(string_list):
        string_list[index] = item.strip()
    return string_list

def validate_tags(cluster_tags):
    """
    The function validate EMR cluster tags with the given required keys
    """
    non_compliance_tags = []
    compliance_tags = []
    clean_required_keys = remove_whitespace_from_list_of_strings(REQUIRED_KEYS)
    for cluster_tag in cluster_tags:
        if cluster_tag["Key"] in clean_required_keys:
            compliance_tags.append(cluster_tag["Key"])
    for key in clean_required_keys:
        if key not in compliance_tags:
            non_compliance_tags.append({
                "non_compliant_key": key,
                "reason": "Key does not exist"
            })
    return non_compliance_tags

def is_tag(event):
    """
    The function checks the type of EMR event.
    """
    if event["detail"]["eventName"] == "RunJobFlow":
        if "elasticmapreduce.amazonaws.com" in event["detail"]["eventSource"]:
            return True
    if event["detail"]["eventName"] == "CreateTags":
        if "ec2.amazonaws.com" in event["detail"]["eventSource"]:
            return True
    if event["detail"]["eventName"] == "AddTags":
        if "elasticmapreduce.amazonaws.com" in event["detail"]["eventSource"]:
            return True
    if event["detail"]["eventName"] == "RemoveTags":
        if "elasticmapreduce.amazonaws.com" in event["detail"]["eventSource"]:
            return True
    return False

def create_non_compliance_message(non_compliance_tags, emr_cluster_id):
    """
    The function creates a non-compliance message to be notified.
    """
    message = "Violation - The following EMR Cluster was detected to be in violation \n\n"
    message += "EMR Cluster ID : " + emr_cluster_id + "\n"
    for violation in non_compliance_tags:
        message += str(violation) + "\n"
    return message

def send_violation(outbound_topic_arn, message, subject, event, context):
    """
    The function sends violation message
    """
    findsnsregion = outbound_topic_arn.split(":")
    snsregion = findsnsregion[3]
    message += '\nAccount: ' + event["account"] + "\n"
    message += "Region: " + event["detail"]["awsRegion"] + "\n"
    message += "\n\n"
    message += "This notification was generated by the Lambda function " + \
        context.invoked_function_arn
    sendclient = boto3.client('sns', region_name=snsregion)
    try:
        sendclient.publish(
            TopicArn=outbound_topic_arn,
            Message=message,
            Subject=subject
        )
    except ClientError as err:
        log.error(err)
        return False

def setup_logging():
    """
    Logging Function.
    Creates a global log object and sets its level.
    """
    global log
    log = logging.getLogger()
    log_levels = {'INFO': 20, 'WARNING': 30, 'ERROR': 40}

    if 'logging_level' in os.environ:
        log_level = os.environ['logging_level'].upper()
        if log_level in log_levels:
            log.setLevel(log_levels[log_level])
        else:
            log.setLevel(log_levels['ERROR'])
            log.error("The logging_level environment variable is not set to INFO, WARNING, or \
                        ERROR.  The log level is set to ERROR")
    else:
        log.setLevel(log_levels['ERROR'])
        log.warning('The logging_level environment variable is not set. The log level is set to \
                    ERROR')
    log.info('Logging setup complete - set to log level ' + str(log.getEffectiveLevel()))
